package main

import (
	"encoding/json"
	"fmt"

	_ "code.wuban.net.cn/odysseus/twitter_syncer/docs"
	// docs are generated by Swag CLI, you have to import them.
	// replace with your own docs folder, usually "github.com/username/reponame/docs"
	//_ "github.com/gofiber/swagger/example/docs"
)

// var client *supabase.Client

// func init() {

// 	var API_URL = "http://43.198.54.207:8000"
// 	var API_KEY = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyAgCiAgICAicm9sZSI6ICJzZXJ2aWNlX3JvbGUiLAogICAgImlzcyI6ICJzdXBhYmFzZS1kZW1vIiwKICAgICJpYXQiOiAxNjQxNzY5MjAwLAogICAgImV4cCI6IDE3OTk1MzU2MDAKfQ.DaYlNEoUrrEn2Ig7tqibS-PHK5vgusbcbo7X36XVt4Q"

// 	for {
// 		cli, err := supabase.NewClient(API_URL, API_KEY, &supabase.ClientOptions{
// 			Schema: "twitter",
// 		})
// 		if err != nil {
// 			slog.Error("supabase.NewClient", "err", err.Error())
// 			return
// 		} else {
// 			client = cli
// 			break
// 		}
// 	}
// }

func AddTaskInsertOrUpdate(req Req) error {

	task, ok, err := Query(req)

	if err != nil {
		return err
	}

	_ = task

	if ok {
		//update
		return AddTaskUpdate(req, task)
	} else {
		//insert
		return Insert(req)
	}

	return nil
}

func AddTaskUpdate(req Req, task Task) error {

	if !task.Follow && req.Follow {
		task.Follow = true
	}

	if !task.Retweet && req.Retweet {
		task.Retweet = true
	}

	res, _, err := client.From("tasks").Update(&struct {
		Retweet bool `json:"retweet"`
		Follow  bool `json:"follow"`
	}{
		Retweet: task.Retweet,
		Follow:  task.Follow,
	}, "", "exact").Eq("project", req.Project).Eq("tweet_id", req.TweetId).Execute()

	_ = res

	return err
}

func StopTaskInsertOrUpdate(req Req) error {

	task, ok, err := Query(req)

	if err != nil {
		return err
	}

	if !ok {
		return fmt.Errorf("can not stop the task,no record")
	}

	return StopTaskUpdate(req, task)

}

func StopTaskUpdate(req Req, task Task) error {

	if !task.FollowStop && req.Follow {
		task.FollowStop = true
	}

	if !task.RetweetStop && req.Retweet {
		task.RetweetStop = true
	}

	res, _, err := client.From("tasks").Update(&struct {
		Retweet bool `json:"retweet_stop"`
		Follow  bool `json:"follow_stop"`
	}{
		Retweet: task.RetweetStop,
		Follow:  task.FollowStop,
	}, "", "exact").Eq("project", req.Project).Eq("tweet_id", req.TweetId).Execute()

	_ = res

	return err

	//return nil
}

func Insert(req Req) error {

	res, _, err := client.From("tasks").Insert(req, true, "", "representation", "").Execute()

	_ = res

	return err

}

func Query(req Req) (Task, bool, error) {

	data, count, err := client.From("tasks").Select("*", "exact", false).Eq("project", req.Project).Eq("tweet_id", req.TweetId).Execute()

	if err != nil {
		return Task{}, false, err
	}

	if count == 0 {
		return Task{}, false, nil
	}

	tasks := make([]Task, 0)

	if err := json.Unmarshal(data, &tasks); err != nil {
		return Task{}, false, err
	}

	var task Task
	if len(tasks) == 1 {
		task = tasks[0]
	} else {
		return Task{}, false, fmt.Errorf("expected raws len ==1, but actually %d", len(tasks))
	}

	return task, true, nil
}

/*

[{"id":3,"created_at":"2024-07-30T09:18:22.110223+00:00","project":"1","tweet_id":"1","like":true,"retweet":true,"like_stop":null,"retweet_stop":null}]

*/

// type Tasks []Task

type Task struct {
	Project     string `json:"project"`
	TweetId     string `json:"tweet_id"`
	Like        bool   `json:"like"`
	Follow      bool   `json:"follow"`
	Retweet     bool   `json:"retweet"`
	LikeStop    bool   `json:"like_stop"`
	FollowStop  bool   `json:"follow_stop"`
	RetweetStop bool   `json:"retweet_stop"`
}

func VerifyFollowerInDb(userId, followerId string) (bool, error) {

	_, count, err := client.From("followers").Select("*", "exact", false).Eq("user_id", userId).Eq("follower_id", followerId).Execute()

	if err != nil {
		return false, err
	}

	return count == 1, nil
}

func VerifyRetweeterInDb(tweetId, retweeter string) (bool, error) {

	_, count, err := client.From("retweeters").Select("*", "exact", false).Eq("tweet_id", tweetId).Eq("retweeter_id", retweeter).Execute()

	if err != nil {
		return false, err
	}

	return count == 1, nil
}
