package main

import (
	"log/slog"
	"time"

	twitterscraper "github.com/imperatrona/twitter-scraper"
)

func newSync() error {

	done := make(<-chan interface{})

	connStream := make(chan taskInterface, 1000)

	idxStream := TaskIdx(done, connStream)

	resStream, err := InitResource(users)
	if err != nil {
		return err
	}

	taskFetchStream, resTailStream := TaskImplement(done, idxStream, resStream)

	ConnectTailTask(done, connStream, taskFetchStream)

	ConnectTailResource(done, resStream, resTailStream)

	select {}

}

func TaskIdx(done <-chan interface{}, inStream <-chan taskInterface) <-chan taskInterface {

	outStream := make(chan taskInterface, 1000)

	return outStream
}

type ScraperTimer struct {
	Scraper *twitterscraper.Scraper
	Timer   time.Timer
}

func InitResource(users []TwitterAccount) (chan ScraperTimer, error) {

	outStream := make(chan ScraperTimer, 1000)

	for _, v := range users {

		scraper, err := InitScraper(v.User, v.PassWd)

		if err != nil {
			return nil, err
		}

		newScraperTimer := ScraperTimer{
			Scraper: scraper,
			Timer:   *time.NewTimer(0),
		}

		outStream <- newScraperTimer

	}

	return outStream, nil
}

func AvailableResource(done <-chan interface{}, inStream <-chan ScraperTimer) <-chan ScraperTimer {

	outStream := make(chan ScraperTimer, 1000)

	go func() {

		for {
			select {
			case <-done:
				return
			case scraper := <-inStream:
				//TODO
				// 1. 携带web服务器的rate limit 做判断；
				// 2. 校验 是否可用
				<-scraper.Timer.C
				//scraper.Timer = *time.NewTimer(30 * time.Second)
				scraper.Timer.Reset(30 * time.Second)
				outStream <- scraper
			}
		}

	}()

	return outStream
}

func TaskImplement(done <-chan interface{}, inTaskStream <-chan taskInterface, inResourceStream <-chan ScraperTimer) (<-chan taskInterface, <-chan ScraperTimer) {

	taskOutStream := make(chan taskInterface, 1000)
	scraperOutStream := make(chan ScraperTimer, 1000)

	go func() {

		for {
			select {
			case <-done:
				return
			case task := <-inTaskStream:

				select {
				case <-done:
					return
				case res := <-inResourceStream:
					if err := task.Fetch(res.Scraper); err != nil {
						slog.Error("task.Fetch", "err", err.Error())
					}
				}
			}
		}
	}()

	return taskOutStream, scraperOutStream
}

func ConnectTailTask(done <-chan interface{}, beginStream chan<- taskInterface, tailStream <-chan taskInterface) {

	go func() {

		for {

			select {
			case <-done:
				return
			case beginStream <- <-tailStream:
			}

		}

	}()

}

func ConnectTailResource(done <-chan interface{}, beginStream chan<- ScraperTimer, tailStream <-chan ScraperTimer) {

	go func() {
		for {
			select {
			case <-done:
				return
			case beginStream <- <-tailStream:
			}
		}
	}()
}
