package main

import (
	b64 "encoding/base64"
	"strings"

	twitterscraper "github.com/imperatrona/twitter-scraper"
)

/*
users {"Avatar":"https://abs.twimg.com/sticky/default_profile_images/default_profile_normal.png","Banner":"","Biography":"","Birthday":"","FollowersCount":61,"FollowingCount":7,"FriendsCount":77,"IsPrivate":false,"IsVerified":false,"Joined":"2022-06-11T15:16:41Z","LikesCount":7,"ListedCount":0,"Location":"","Name":"Wade","PinnedTweetIDs":[],"TweetsCount":55,"URL":"https://twitter.com/Wade_Leeeee","UserID":"VXNlcjoxNTM1NjQyMTUyNTY2MjU5NzEy","Username":"Wade_Leeeee","Website":"","Sensitive":false,"Following":false,"FollowedBy":false}
k 0 Wade_Leeeee VXNlcjoxNTM1NjQyMTUyNTY2MjU5NzEy
User:1535642152566259712

users {"Avatar":"https://pbs.twimg.com/profile_images/1787913259074691072/XioiWSMF_normal.jpg","Banner":"https://pbs.twimg.com/profile_banners/1568295462486016008/1715106782","Biography":"Positive mind, positive vibes, positive life ✨","Birthday":"","FollowersCount":47,"FollowingCount":0,"FriendsCount":548,"IsPrivate":false,"IsVerified":false,"Joined":"2022-09-09T17:49:17Z","LikesCount":0,"ListedCount":0,"Location":"South Kensington, UK","Name":"Aurora","PinnedTweetIDs":[],"TweetsCount":21,"URL":"https://twitter.com/Shae85083106","UserID":"VXNlcjoxNTY4Mjk1NDYyNDg2MDE2MDA4","Username":"Shae85083106","Website":"","Sensitive":false,"Following":false,"FollowedBy":false}
k 1 Shae85083106 VXNlcjoxNTY4Mjk1NDYyNDg2MDE2MDA4
User:1568295462486016008
*/
type Users struct {
	Profiles []Profile
	Current  string
	Next     string
	Error    error
}

type Profile struct {
	UserIdAsNumber string
	*twitterscraper.Profile
}

type NewTask[T FollowTask | RetweetTask] struct {
	Task T
	Init bool
}

type FollowTask struct {
	URL    string
	UserId string
	Next   string

	Res     Users
	Scraper *twitterscraper.Scraper
}

type taskInterface interface {
	Fetch(scraper *twitterscraper.Scraper) error
	SetScraper(scraper *twitterscraper.Scraper)
	SetScraperNil()
}

// func (f *FollowTask) URl() string {
// 	return f.URL
// }

func (f *FollowTask) ID() string {
	return f.UserId
}

func (f *FollowTask) SetScraper(scraper *twitterscraper.Scraper) {
	f.Scraper = scraper
}

func (f *FollowTask) SetScraperNil() {
	f.Scraper = nil
}

func (f *FollowTask) Fetch(scraper *twitterscraper.Scraper) error {
	users, newNext, err := scraper.FetchFollowers(f.UserId, 20, f.Next)

	if err != nil {
		return err
	}

	usersWithUserNumber := make([]Profile, 0, len(users))

	for _, v := range users {
		sDec, _ := b64.StdEncoding.DecodeString(v.UserID)
		userId, _ := strings.CutPrefix(string(sDec), "User:")
		item := Profile{
			UserIdAsNumber: userId,
			Profile:        v,
		}
		usersWithUserNumber = append(usersWithUserNumber, item)
	}

	res := Users{
		Profiles: usersWithUserNumber,
		Current:  f.Next,
		Next:     newNext,
	}

	f.Res = res

	return nil

}

type RetweetTask struct {
	URL     string
	TweetId string
	Next    string
	Scraper *twitterscraper.Scraper
}

// func (r *RetweetTask) URl() string {
// 	return r.URL
// }

func (f *RetweetTask) ID() string {
	return f.TweetId
}

func (f *RetweetTask) SetScraper(scraper *twitterscraper.Scraper) {
	f.Scraper = scraper
}

func (f *RetweetTask) Fetch(scraper *twitterscraper.Scraper) error {

	return scraper.RetweetsUsers(f.TweetId)

}

func (f *RetweetTask) SetScraperNil() {
	f.Scraper = nil
}

type TwitterAccount struct {
	User   string
	PassWd string
}

var users []TwitterAccount = []TwitterAccount{
	TwitterAccount{
		User:   "Wade_Leeeee",
		PassWd: "923881393time",
	},
}
